@@ -1,25 +1,22 @@
-/* LogJam, a GTK LiveJournal client.
- * Copyright (C) 2000,2001 Evan Martin <evan@livejournal.com>
- * vim:ts=4:sw=4:
+/* logjam - a GTK client for LiveJournal.
+ * Copyright (C) 2000-2002 Evan Martin <evan@livejournal.com>
  *
- * $Id: history.c,v 1.6 2001/11/10 00:44:13 martine Exp $
+ * vim: tabstop=4 shiftwidth=4 noexpandtab :
+ * $Id: history.c,v 1.19 2002/10/03 01:11:43 martine Exp $
  */
 
 #include <gtk/gtk.h>
 #include <time.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
-#include <libhalfgnome/halfgnome.h>
-#include <libhalfgnome/spawn.h>
-
-#include "security.h"
-#include "meta.h"
 #include "util.h"
-#include "dotconf.h"
+#include "conf.h"
 #include "network.h"
-#include "gtkspell.h"
+#include "spawn.h"
 
+#if 0
 typedef struct {
 	GtkWidget *win;
 	GtkWidget *daylabel;
@@ -34,7 +31,7 @@ typedef struct {
 #define MARKMONTH(h, year, mon) (h)->markmonths[((year)-(h)->firstyear)*12+((mon)-1)]
 #define MARKMONTH_CUR(h) MARKMONTH(h, h->year, h->mon)
 
-typedef struct {
+/*typedef struct {
 	GtkWidget *win;
 	GtkWidget *esubject;
 	GtkWidget *eentry;
@@ -44,10 +41,10 @@ typedef struct {
 	int itemid;
 
 	history_calendar_dlg *hcdlg;
-} history_item_dlg;
+} history_item_dlg;*/
 
 static gboolean getdaycounts_run(history_calendar_dlg* hcdlg);
-extern void popcalendar_run(GtkWidget *parent, guint date[], int markfirstyear, guint32 marks[]);
+void popcalendar_run(GtkWidget *parent, guint date[], int markfirstyear, guint32 marks[]);
 
 static void change_day(history_calendar_dlg *hcdlg);
 
@@ -57,9 +54,9 @@ static void hc_row_selected(GtkCList *list, gint row, gint col,
 static gint hc_list_click_cb(GtkCList *list, GdkEventButton *be, 
 		history_calendar_dlg *hcdlg);
 static void hc_edit_cb(GtkWidget *w, history_calendar_dlg *hcdlg);
-static void hc_web_cb(GtkWidget *w, history_calendar_dlg *hcdlg);
+//static void hc_web_cb(GtkWidget *w, history_calendar_dlg *hcdlg);
 
-static gint hi_dialog_run(GtkWidget *parent, int itemid);
+/* fixme static gint hi_dialog_run(GtkWidget *parent, int itemid);*/
 
 static void
 free_dlg_cb(GtkWidget *w, history_calendar_dlg *hcdlg) {
@@ -122,8 +119,8 @@ make_contents(history_calendar_dlg *hcdlg) {
 
 	button = gtk_button_new();
 	hcdlg->daylabel = gtk_label_new("");
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(popup_calendar), hcdlg);
+	g_signal_connect(G_OBJECT(button), "clicked",
+			G_CALLBACK(popup_calendar), hcdlg);
 	gtk_container_add(GTK_CONTAINER(button), hcdlg->daylabel);
 
 	scrollwin = gtk_scrolled_window_new (NULL, NULL); 
@@ -132,15 +129,15 @@ make_contents(history_calendar_dlg *hcdlg) {
 	hcdlg->evlist = gtk_clist_new_with_titles(2, titles);
 	gtk_clist_set_selection_mode(GTK_CLIST(hcdlg->evlist),
 			GTK_SELECTION_BROWSE);
-	gtk_signal_connect(GTK_OBJECT(hcdlg->evlist), "select-row", 
-			GTK_SIGNAL_FUNC(hc_row_selected), hcdlg);
-	gtk_signal_connect(GTK_OBJECT(hcdlg->evlist), "unselect-row", 
-			GTK_SIGNAL_FUNC(hc_row_selected), hcdlg);
-	gtk_signal_connect(GTK_OBJECT(hcdlg->evlist), "button_press_event",
-			GTK_SIGNAL_FUNC(hc_list_click_cb), hcdlg);
+	g_signal_connect(G_OBJECT(hcdlg->evlist), "select-row", 
+			G_CALLBACK(hc_row_selected), hcdlg);
+	g_signal_connect(G_OBJECT(hcdlg->evlist), "unselect-row", 
+			G_CALLBACK(hc_row_selected), hcdlg);
+	g_signal_connect(G_OBJECT(hcdlg->evlist), "button_press_event",
+			G_CALLBACK(hc_list_click_cb), hcdlg);
 	gtk_clist_column_titles_passive(GTK_CLIST(hcdlg->evlist));
-	gtk_clist_set_column_width(GTK_CLIST(hcdlg->evlist), 0, 
-			gdk_string_width(hcdlg->evlist->style->font, "00:00a"));
+	/* fixme gtk2 gtk_clist_set_column_width(GTK_CLIST(hcdlg->evlist), 0, 
+			gdk_string_width(hcdlg->evlist->style->font, "00:00a"));*/
 	titles[0] = NULL;
 	titles[1] = "[select a day]";
 	gtk_clist_append(GTK_CLIST(hcdlg->evlist), titles);
@@ -159,14 +156,14 @@ make_buttonbox(history_calendar_dlg *hcdlg) {
 
 	box = lj_dialog_buttonbox_new();
 	button = gtk_button_new_with_label(" Edit... ");
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(hc_edit_cb), hcdlg);
+	g_signal_connect(G_OBJECT(button), "clicked",
+			G_CALLBACK(hc_edit_cb), hcdlg);
 	lj_dialog_buttonbox_add(box, button);
 
-	button = web_button(hcdlg->win, "View...");
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(hc_web_cb), hcdlg);
-	lj_dialog_buttonbox_add(box, button);
+	/*button = web_button(hcdlg->win, "View...");
+	g_signal_connect(G_OBJECT(button), "clicked",
+			G_CALLBACK(hc_web_cb), hcdlg);
+	lj_dialog_buttonbox_add(box, button);*/
 	return box;
 }
 
@@ -176,8 +173,8 @@ void history_dialog(GtkWidget *mainwin) {
 	hcdlg = g_new0(history_calendar_dlg, 1);
 
 	hcdlg->win = lj_dialog_new(mainwin, "History Overview", 400, -1);
-	gtk_signal_connect(GTK_OBJECT(hcdlg->win), "destroy",
-			GTK_SIGNAL_FUNC(free_dlg_cb), hcdlg);
+	g_signal_connect(G_OBJECT(hcdlg->win), "destroy",
+			G_CALLBACK(free_dlg_cb), hcdlg);
 
 	hcdlg->bbox = make_buttonbox(hcdlg);
 	lj_dialog_set_contents_buttonbox(hcdlg->win,
@@ -230,15 +227,16 @@ daycounts_hash_cb(gpointer key, gpointer value, gpointer data) {
 
 static gboolean
 getdaycounts_run(history_calendar_dlg* hcdlg) {
-	GHashTable *request, *result;
+	NetRequest *request;
+	NetResult  *result;
 	
 	request = net_request_new("getdaycounts");
 	
 	result = net_request_run(hcdlg->win, "Loading calendar...", request);
-	hash_destroy(request);
+	net_request_free(request);
 
-	if (!net_request_succeeded(result)) {
-		hash_destroy(result);
+	if (!net_result_succeeded(result)) {
+		net_result_free(result);
 		gtk_widget_destroy(hcdlg->win);
 		return FALSE;
 	}
@@ -253,7 +251,7 @@ getdaycounts_run(history_calendar_dlg* hcdlg) {
 	/* then read all of the values from the hash. */
 	g_hash_table_foreach(result, daycounts_hash_cb, hcdlg);
 
-	hash_destroy(result);
+	net_result_free(result);
 	return TRUE;
 }
 
@@ -266,34 +264,33 @@ change_day(history_calendar_dlg *hcdlg) {
 	gint row;
 	char *append[2];
 
-	GHashTable *request, *result;
+	NetRequest *request;
+	NetResult  *result;
 	
 	if (hcdlg->firstyear == 0) return; /* this can happen if they cancelled the getdaycounts */
 
 	request = net_request_new("getevents");
 
-	g_hash_table_insert(request, g_strdup("truncate"), g_strdup("50"));
-	g_hash_table_insert(request, g_strdup("prefersubject"), g_strdup("1"));
-	g_hash_table_insert(request, g_strdup("noprops"), g_strdup("1"));
-	g_hash_table_insert(request, g_strdup("selecttype"), g_strdup("day"));
-	g_hash_table_insert(request, g_strdup("lineendings"), g_strdup("dots"));
+	net_request_seti(request,  "truncate",      50);
+	net_request_seti(request,  "prefersubject", 1);
+	net_request_seti(request,  "noprops",       1);
+	net_request_copys(request, "selecttype",    "day");
+	net_request_copys(request, "lineendings",   "dots");
 
-	g_hash_table_insert(request, g_strdup("year"), 
-			g_strdup_printf("%d", hcdlg->year));
-	g_hash_table_insert(request, g_strdup("month"), 
-			g_strdup_printf("%d", hcdlg->mon));
-	g_hash_table_insert(request, g_strdup("day"), 
-			g_strdup_printf("%d", hcdlg->day));
+	net_request_seti(request, "year", hcdlg->year);
+	net_request_seti(request, "month", hcdlg->mon);
+	net_request_seti(request, "day", hcdlg->day);
 
 	result = net_request_run(hcdlg->win, "Loading day calendar...", request);
-	hash_destroy(request);
+	net_request_free(request);
 
-	if (!net_request_succeeded(result)) {
-		hash_destroy(result);
+	if (!net_result_succeeded(result)) {
+		net_result_free(result);
 		return;
 	}
 
-	str = g_hash_table_lookup(result, "events_count");
+	/* it would have been better if net_result_geti used a GError */
+	str = net_result_get(result, "events_count");
 	if (str == NULL) return;
 	count = atoi(str);
 
@@ -314,14 +311,14 @@ change_day(history_calendar_dlg *hcdlg) {
 
 	for (i = 1; i < count+1; i++) {
 		sprintf(key, "events_%d_event", i);
-		event = g_hash_table_lookup(result, key);
+		event = net_result_get(result, key);
 
 		sprintf(key, "events_%d_eventtime", i);
-		str = g_hash_table_lookup(result, key);
+		str = net_result_get(result, key);
 		sscanf(str+11, "%2d:%2d", &hour, &minute);
 
 		sprintf(key, "events_%d_itemid", i);
-		str = g_hash_table_lookup(result, key);
+		str = net_result_get(result, key);
 		if (str == NULL) continue;
 		itemid = atoi(str);
 
@@ -335,7 +332,7 @@ change_day(history_calendar_dlg *hcdlg) {
 	gtk_clist_set_column_width(GTK_CLIST(hcdlg->evlist), 0, 
 			gtk_clist_optimal_column_width(GTK_CLIST(hcdlg->evlist), 0));
 
-	hash_destroy(result);
+	net_result_free(result);
 }
 
 static void 
@@ -365,7 +362,7 @@ hc_show_current(history_calendar_dlg *hcdlg) {
 	if (itemid < 0)
 		return;
 
-	hi_dialog_run(hcdlg->win, itemid);
+	/* fixme hi_dialog_run(hcdlg->win, itemid); */
 }
 
 static gint
@@ -392,7 +389,7 @@ hc_edit_cb(GtkWidget *w, history_calendar_dlg *hcdlg) {
 	hc_show_current(hcdlg);
 }
 
-static void
+/*static void
 history_delete_selected(history_item_dlg *hidlg) {
 	GHashTable *request, *result;
 	
@@ -411,19 +408,20 @@ history_delete_selected(history_item_dlg *hidlg) {
 		gtk_widget_destroy(hidlg->win);
 	}
 	hash_destroy(result);
-}
+}*/
 
-static void
+/*static void
 hc_web_cb(GtkWidget *w, history_calendar_dlg *hcdlg) {
 	char *spawn;
 
 	spawn = g_strdup_printf("%s/talkread.bml?itemid=%d", 
-			conf.ljserver,
+			conf_cur_server()->url,
 			hc_current_itemid(hcdlg));
-	halfgnome_spawn_url(spawn);
+	spawn_url(spawn);
 	g_free(spawn);
-}
+}*/
 
+fixme 
 static void
 hi_load_metadata(history_item_dlg *hidlg, GHashTable *result) {
 	GHashTable *metadata;
@@ -525,7 +523,7 @@ hi_load_result(history_item_dlg *hidlg, GHashTable* result) {
 	insert_point = 0;
 	gtk_editable_insert_text(GTK_EDITABLE(hidlg->eentry), 
 			value, strlen(value), &insert_point);
-	gtkspell_check_all(GTK_TEXT(hidlg->eentry)); 
+	/* fixme gtkspell gtkspell_check_all(GTK_TEXT(hidlg->eentry)); */
 	gtk_editable_set_position(GTK_EDITABLE(hidlg->eentry), 0);
 
 	/* subject */
@@ -579,7 +577,7 @@ hi_save_cb(GtkWidget *w, history_item_dlg *hidlg) {
 
 static void
 hi_delete_cb(GtkWidget *w, history_item_dlg *hidlg) {
-	if (conf.confirmdelete)
+	if (conf.options.confirmdelete)
 		if (!lj_confirm(hidlg->win, "Delete?", "Delete this entry?"))
 			return;
 
@@ -595,7 +593,8 @@ hi_dialog_date_hbox(history_item_dlg *hidlg) {
 	hbox = gtk_hbox_new(FALSE, 0); {
 		/* year */
 		hidlg->eyear = gtk_entry_new_with_max_length(4);
-		twocharwidth = gdk_string_width(hidlg->eyear->style->font, "00");
+		/* fixme gtk2 twocharwidth = gdk_string_width(hidlg->eyear->style->font, "00");*/
+		twocharwidth = 20;
 		gtk_box_pack_start(GTK_BOX(hbox), hidlg->eyear, FALSE, FALSE, 0);
 		gtk_widget_set_usize(GTK_WIDGET(hidlg->eyear), twocharwidth*2 + 5, -1);
 		/* dash */ 
@@ -641,7 +640,7 @@ hi_dialog_run(GtkWidget *parent, int itemid) {
 	history_item_dlg hidlg_actual = { 0 };
 	history_item_dlg *hidlg = &hidlg_actual;
 	GtkWidget *vbox, *button;
-	GHashTable *result;
+	NetResult *result;
 
 	hidlg->itemid = itemid;
 	result = hi_request_run(hidlg, itemid);
@@ -649,9 +648,9 @@ hi_dialog_run(GtkWidget *parent, int itemid) {
 		return 0;
 
 	hidlg->win = lj_dialog_new(parent, "History Item", 300, 300);
-	geometry_tie(hidlg->win, &conf.hiwin_geom);
-	gtk_signal_connect(GTK_OBJECT(hidlg->win), "destroy",
-			GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+	geometry_tie(hidlg->win, GEOM_HISTORY_ITEM);
+	g_signal_connect(G_OBJECT(hidlg->win), "destroy",
+			G_CALLBACK(gtk_main_quit), NULL);
 
 	vbox = gtk_vbox_new(FALSE, 5); {
 		GtkWidget *hbox, *scroll;
@@ -692,7 +691,7 @@ hi_dialog_run(GtkWidget *parent, int itemid) {
 					GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 
 			hidlg->eentry = gtk_text_new(NULL, NULL);
-			gtkspell_attach(GTK_TEXT(hidlg->eentry));
+			/* fixme gtkspell gtkspell_attach(GTK_TEXT(hidlg->eentry)); */
 			gtk_text_set_editable(GTK_TEXT(hidlg->eentry), TRUE);
 			gtk_text_set_word_wrap(GTK_TEXT(hidlg->eentry), TRUE);
 			gtk_container_add(GTK_CONTAINER(scroll), hidlg->eentry);
@@ -706,13 +705,13 @@ hi_dialog_run(GtkWidget *parent, int itemid) {
 	lj_dialog_set_contents(hidlg->win, vbox);
 
 	button = gtk_button_new_with_label("  Save Changes  ");
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(hi_save_cb), (gpointer) hidlg);
+	g_signal_connect(G_OBJECT(button), "clicked",
+			G_CALLBACK(hi_save_cb), (gpointer) hidlg);
 	lj_dialog_add_button(hidlg->win, button);
 
 	button = gtk_button_new_with_label("  Delete  ");
-	gtk_signal_connect(GTK_OBJECT(button), "clicked",
-			GTK_SIGNAL_FUNC(hi_delete_cb), (gpointer) hidlg);
+	g_signal_connect(G_OBJECT(button), "clicked",
+			G_CALLBACK(hi_delete_cb), (gpointer) hidlg);
 	lj_dialog_add_button(hidlg->win, button);
 	
 	lj_dialog_add_close(hidlg->win);
@@ -727,9 +726,39 @@ hi_dialog_run(GtkWidget *parent, int itemid) {
 
 	return 0;
 }
+#endif
 
-void
-history_load_latest(GtkWidget *mainwin) {
-	hi_dialog_run(mainwin, -1); /* load itemid -1: the last item! */
+Entry*
+history_load_itemid(GtkWidget *win, int itemid) {
+	NetRequest *request;
+	NetResult *result;
+	Entry *entry;
+
+	/* FIXME cache: hit cache first. */
+	request = net_request_new("getevents");
+	net_request_copys(request, "selecttype", "one");
+	net_request_seti(request, "itemid", itemid);
+	net_request_copys(request, "lineendings", "unix");
+	if (conf.usejournal)
+		net_request_copys(request, "usejournal", conf.usejournal);
+
+	result = net_request_run(win, _("Loading Entry..."), request);
+	net_request_free(request);
+
+	if (!net_result_succeeded(result)) {
+		net_result_free(result);
+		return NULL;
+	}
+
+	entry = entry_from_result(result, 1);
+	net_result_free(result);
+
+	return entry;
+}
+
+Entry*
+history_load_latest(GtkWidget *win) {
+	/* FIXME cache: hit cache first. */
+	return history_load_itemid(win, -1);
 }
 
